@@ -52,15 +52,14 @@
 
 #include <libxfce4mcs/mcs-common.h>
 #include <libxfce4mcs/mcs-manager.h>
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/util.h>
+#include <libxfce4util/libxfce4util.h>
 #include <xfce-mcs-manager/manager-plugin.h>
 #include <libxfcegui4/libxfcegui4.h>
 
-#include "inline-icon.h"
 #include "plugins/gtk_common/gtk_common.h"
 
-#define RCDIR    "settings"
+#define RCDIR "mcs_settings"
+#define OLDRCDIR "settings"
 #define CHANNEL1 "settings"
 #define RCFILE1 "gtk.xml"
 #define CHANNEL2 "keyboard"
@@ -72,7 +71,6 @@
 #define OFF 0
 #define ALL -1
 
-#define DEFAULT_ICON_SIZE 48
 #define MAX_ELEMENTS_BEFORE_SCROLLING 6
 
 static void create_channel(McsPlugin * mcs_plugin);
@@ -281,7 +279,7 @@ static void read_themes(Itf * dialog)
             GtkRequisition rectangle;
             gtk_widget_size_request(GTK_WIDGET(tree_view), &rectangle);
             gtk_widget_set_size_request(dialog->theme_swindow, -1, rectangle.height);
-            gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+            gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
         }
         i++;
     }
@@ -417,7 +415,6 @@ static void cb_blinktime_changed(GtkWidget * dialog, gpointer user_data)
 Itf *create_keyboard_dialog(McsPlugin * mcs_plugin)
 {
     Itf *dialog;
-    GdkPixbuf *icon;
 
     dialog = g_new(Itf, 1);
 
@@ -425,9 +422,7 @@ Itf *create_keyboard_dialog(McsPlugin * mcs_plugin)
 
     dialog->keyboard_dialog = gtk_dialog_new();
 
-    icon = inline_icon_at_size(default_icon_data, 32, 32);
-    gtk_window_set_icon(GTK_WINDOW(dialog->keyboard_dialog), icon);
-    g_object_unref(icon);
+    gtk_window_set_icon(GTK_WINDOW(dialog->keyboard_dialog), mcs_plugin->icon);
 
     gtk_window_set_title(GTK_WINDOW(dialog->keyboard_dialog), _("Keyboard Preferences"));
     gtk_window_set_default_size(GTK_WINDOW(dialog->keyboard_dialog), 440, 200);
@@ -436,7 +431,8 @@ Itf *create_keyboard_dialog(McsPlugin * mcs_plugin)
     dialog->dialog_vbox1 = GTK_DIALOG(dialog->keyboard_dialog)->vbox;
     gtk_widget_show(dialog->dialog_vbox1);
 
-    dialog->dialog_header = create_header(icon, _("Keyboard Preferences"));
+    dialog->dialog_header = xfce_create_header(mcs_plugin->icon,
+                                               _("Keyboard Preferences"));
     gtk_widget_show(dialog->dialog_header);
     gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->dialog_header, FALSE, TRUE, 0);
 
@@ -488,25 +484,25 @@ Itf *create_keyboard_dialog(McsPlugin * mcs_plugin)
     gtk_widget_show(dialog->table1);
     gtk_box_pack_start(GTK_BOX(dialog->vbox4), dialog->table1, TRUE, TRUE, 0);
 
-    dialog->label2 = small_label(_("Short"));
+    dialog->label2 = xfce_create_small_label(_("Short"));
     gtk_widget_show(dialog->label2);
     gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label2, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label2), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label2), 1, 0.5);
 
-    dialog->label3 = small_label(_("Long"));
+    dialog->label3 = xfce_create_small_label(_("Long"));
     gtk_widget_show(dialog->label3);
     gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label3, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label3), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label3), 0, 0.5);
 
-    dialog->label4 = small_label(_("Slow"));
+    dialog->label4 = xfce_create_small_label(_("Slow"));
     gtk_widget_show(dialog->label4);
     gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label4, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label4), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label4), 1, 0.5);
 
-    dialog->label5 = small_label(_("Fast"));
+    dialog->label5 = xfce_create_small_label(_("Fast"));
     gtk_widget_show(dialog->label5);
     gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label5, 2, 3, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label5), GTK_JUSTIFY_LEFT);
@@ -562,13 +558,13 @@ Itf *create_keyboard_dialog(McsPlugin * mcs_plugin)
     gtk_label_set_justify(GTK_LABEL(dialog->label9), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label9), 0, 0.5);
 
-    dialog->label10 = small_label(_("Slow"));
+    dialog->label10 = xfce_create_small_label(_("Slow"));
     gtk_widget_show(dialog->label10);
     gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label10, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label10), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label10), 1, 0.5);
 
-    dialog->label11 = small_label(_("Fast"));
+    dialog->label11 = xfce_create_small_label(_("Fast"));
     gtk_widget_show(dialog->label11);
     gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label11, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label11), GTK_JUSTIFY_LEFT);
@@ -624,43 +620,31 @@ static void setup_dialog(Itf *itf)
     g_signal_connect(G_OBJECT(itf->checkblink), "toggled", G_CALLBACK(cb_checkblink_changed), itf);
     g_signal_connect(G_OBJECT(itf->hscale3), "value_changed", (GCallback) cb_blinktime_changed, itf);
 
-    gtk_window_set_position(GTK_WINDOW(itf->keyboard_dialog), GTK_WIN_POS_CENTER);
+    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->keyboard_dialog));
     gtk_widget_show(itf->keyboard_dialog);
 }
 
 McsPluginInitResult
 mcs_plugin_init(McsPlugin *mcs_plugin)
 {
-#if 0
-#ifdef ENABLE_NLS
-    /* This is required for UTF-8 at least - Please don't remove it */
-    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
-    textdomain (GETTEXT_PACKAGE);
-#endif
-#else
     /* This is required for UTF-8 at least - Please don't remove it */
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-#endif
 
     create_channel(mcs_plugin);
     mcs_plugin->plugin_name = g_strdup(PLUGIN_NAME);
     mcs_plugin->caption = g_strdup(_("Keyboard"));
     mcs_plugin->run_dialog = run_dialog;
-    mcs_plugin->icon = inline_icon_at_size(default_icon_data,
-		    DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+    mcs_plugin->icon = xfce_themed_icon_load ("xfce4-keyboard", 48);
     mcs_manager_notify(mcs_plugin->manager, CHANNEL1);
 
-    return(MCS_PLUGIN_INIT_OK);
+    return MCS_PLUGIN_INIT_OK;
 }
 
 static void
 create_channel(McsPlugin *mcs_plugin)
 {
     McsSetting *setting;
-    gchar *rcfile;
+    gchar *rcfile, *path;
 #ifdef USE_XF86MISC
     int major, minor;
 #endif
@@ -669,14 +653,49 @@ create_channel(McsPlugin *mcs_plugin)
     int xkbopcode, xkbevent, xkberror;
 #endif
     
-    rcfile = xfce_get_userfile(RCDIR, RCFILE1, NULL);
-    mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL1, rcfile);
+    path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);
+    rcfile = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, path);
+
+    if (!rcfile)
+    {
+        rcfile = xfce_get_userfile(OLDRCDIR, RCFILE1, NULL);
+    }
+
+    if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
+    {
+        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL1, 
+                                          rcfile);
+    }
+    else
+    {
+        mcs_manager_add_channel(mcs_plugin->manager, CHANNEL1);
+    }
+    
+    g_free(path);
     g_free(rcfile);
 
-    rcfile = xfce_get_userfile(RCDIR, RCFILE2, NULL);
-    mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL2, rcfile);
+    path = g_build_filename ("xfce4", RCDIR, RCFILE2, NULL);
+    rcfile = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, path);
+
+    if (!rcfile)
+    {
+        rcfile = xfce_get_userfile(OLDRCDIR, RCFILE2, NULL);
+    }
+
+    if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
+    {
+        mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL2, 
+                                          rcfile);
+    }
+    else
+    {
+        mcs_manager_add_channel(mcs_plugin->manager, CHANNEL2);
+    }
+    
+    g_free(path);
     g_free(rcfile);
 
+
     setting = mcs_manager_setting_lookup(mcs_plugin->manager,
 		    "Gtk/KeyThemeName", CHANNEL1);
     if (setting) {
@@ -797,22 +816,28 @@ create_channel(McsPlugin *mcs_plugin)
     g_warning("This build doesn't include support for Xkb extension");
 #endif
 #endif
+    
+    write_options (mcs_plugin);
 }
 
 static gboolean
 write_options(McsPlugin *mcs_plugin)
 {
     gboolean result;
-    gchar *rcfile;
+    gchar *rcfile, *path;
 
-    rcfile = xfce_get_userfile(RCDIR, RCFILE1, NULL);
+    path = g_build_filename ("xfce4", RCDIR, RCFILE1, NULL);
+    rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
     result = mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL1,
 		    rcfile);
+    g_free(path);
     g_free(rcfile);
 
-    rcfile = xfce_get_userfile(RCDIR, RCFILE2, NULL);
+    path = g_build_filename ("xfce4", RCDIR, RCFILE2, NULL);
+    rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
     result &= mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL2,
 		    rcfile);
+    g_free(path);
     g_free(rcfile);
 
     return(result);
@@ -826,6 +851,8 @@ run_dialog(McsPlugin *mcs_plugin)
     if (!is_running) {
 	is_running = TRUE;
 
+    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+
 	dialog = create_keyboard_dialog(mcs_plugin);
     	setup_dialog(dialog);
     }
