@@ -13,6 +13,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
+        GNOME Control Center - (c) 1998 Jonathan Blandford <jrb@redhat.com>
         xfce4 mcs plugin   - (c) 2002 Olivier Fourdan
 
  */
@@ -56,6 +57,7 @@
 #define DBL_CLICK_MIN   100
 #define DBL_CLICK_MAX   2000
 #define DEFAULT_ICON_SIZE 48
+#define DEFAULT_PTR_MAP_SIZE 128
 
 static void create_channel(McsPlugin * mcs_plugin);
 static gboolean write_options(McsPlugin * mcs_plugin);
@@ -69,6 +71,7 @@ static int acceleration = DEFAULT_ACCELERATION;
 static int threshold = DEFAULT_THRESHOLD;
 static int denominator = DEFAULT_DENOMINATOR;
 
+
 typedef struct _Itf Itf;
 struct _Itf
 {
@@ -136,46 +139,43 @@ static void get_mouse_values(int *accel_return, int *denom_return, int *thresh_r
 
 static void set_mouse_values(gboolean right_handed, int accel, int threshold)
 {
-    unsigned char map[7] = { '\0','\0','\0','\0','\0','\0','\0' };
-    unsigned int buttons = 0;
+    unsigned char *buttons;
+    gint n_buttons, i;
+    gint idx_1 = 0, idx_3 = 1;
 
     gdk_flush();
     gdk_error_trap_push();
 
     XChangePointerControl(GDK_DISPLAY(), True, True, accel, denominator, threshold);
-    buttons = XGetPointerMapping(GDK_DISPLAY(), map, 7);
 
-    if(right_handed)
+    buttons = g_alloca (DEFAULT_PTR_MAP_SIZE);
+    n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, DEFAULT_PTR_MAP_SIZE);
+
+    if (n_buttons > DEFAULT_PTR_MAP_SIZE) 
     {
-        if(buttons > 2)
+        buttons = g_alloca (n_buttons);
+        n_buttons = XGetPointerMapping (GDK_DISPLAY (), buttons, n_buttons);
+    }
+
+    for (i = 0; i < n_buttons; i++)
+    {
+        if (buttons[i] == 1)
         {
-            map[0] = 1;
-            map[1] = 2;
-            map[2] = 3;
+            idx_1 = i;
         }
-        else
+        else if (buttons[i] == ((n_buttons < 3) ? 2 : 3))
         {
-            map[0] = 2;
-            map[1] = 1;
+            idx_3 = i;
         }
     }
-    else
+
+    if ((!right_handed && (idx_1 < idx_3)) || (right_handed && (idx_1 > idx_3)))
     {
-        if(buttons > 2)
-        {
-            map[0] = 3;
-            map[1] = 2;
-            map[2] = 1;
-        }
-        else
-        {
-            map[0] = 1;
-            map[1] = 2;
-        }
+        buttons[idx_1] = ((n_buttons < 3) ? 2 : 3);
+        buttons[idx_3] = 1;
     }
-    XSetPointerMapping(GDK_DISPLAY(), map, buttons);
-    gdk_flush();
-    gdk_error_trap_pop();
+
+    XSetPointerMapping (GDK_DISPLAY (), buttons, n_buttons);
 }
 
 static void cb_dialog_response(GtkWidget * dialog, gint response_id)
